/*****************************************************************************

   This file is part of Virtual VCS, the Atari 2600 Emulator
   ===================================================

   Copyright 1996 Alex Hornby. For contributions see the file CREDITS.
   Modified 1996 by Daniel Boris

   This software is distributed under the terms of the GNU General Public
   License. This is free software with ABSOLUTELY NO WARRANTY.

   See the file COPYING for Details.

   $Id: display.c,v 1.23 1996/03/21 15:52:38 alex Exp $
******************************************************************************/

/*
   Display handling code.
*/

#include <stdio.h>
#include <stdlib.h>

#include "types.h"
#include "config.h"
#include "vmachine.h"
#include "address.h"
#include "files.h"
#include "colours.h"
#include "keyboard.h"
#include "resource.h"

#include "gpdef.h"
#include "gpstdlib.h"
#include "gpgraphic.h"
#include "tiasound.h"
/*#include "dos.h"*/

extern GPDRAWSURFACE gpDraw[2]; /*the main screen, well 2, for a double buffer*/
extern GPDRAWSURFACE vidbuf; /*our hidden screen*/
extern GP_HPALETTE h_pal;

extern int ExKey;

extern unsigned short nflip;
extern unsigned short frames_count; /* FPS Counter */
extern unsigned char current_screen;

extern unsigned short soundbuffer1[SOUND_BUFFERSIZE], soundbuffer2[SOUND_BUFFERSIZE];

#define VGA256		0x13
#define TEXT_MODE	0x03
#define PALETTE_MASK      0x3c6
#define PALETTE_REGISTER  0x3c8
#define PALETTE_DATA	  0x3c9

/*unsigned char far *screen=(char far *)0xA0000000L;*/

int screen_num;
unsigned long white,black;

/* The pointer to the graphics buffer */
BYTE *vscreen;

/* The width and height of the buffer */
int vwidth,vheight;

char coltable[256];

/* The refresh skipping counter */
int tv_counter=0;
int redraw_flag=1;

GP_PALETTEENTRY palette_tmp[256];

void set_palette(int index,BYTE red,BYTE green,BYTE blue)
{
/*
	outportb(0x3c6,0xff);
	outportb(0x3c8,index);
	outportb(0x3c9,red);
	outportb(0x3c9,green);
	outportb(0x3c9,blue);
*/
}

void setmode(int vmode)
{
/*
	union REGS regs;

	regs.h.ah = 0;
	regs.h.al = vmode;
	int86(0x10, &regs, &regs);
*/
}


/** PUTIMAGE: macro copying image buffer into a window ********/
void put_image(void)
{
	int i, xx, yy;
	/*
		memcpy(&screen[0],&vscreen[0],64000);
	*/
	/*GpTextOut(NULL, &vidbuf.ptbuffer, 0, 0, "prova", 1);*/
	/*vscreen[0] = 0;*/
/*	memcpy(&vidbuf.ptbuffer[0],&vscreen[0],64000);*/
/*	vidbuf.ptbuffer[0] = 0;*/
/*	for (i=0; i<64000; i++) {
		vidbuf.ptbuffer[i] = vscreen[i];
	}
*/
	/* Copy rotated */
/*
	i=0;
	for (xx=0; xx<320; xx++) {
		for (yy=199; yy>=0; yy--, i++) {
			vidbuf.ptbuffer[i] = vscreen[yy*320+xx];
		}
		i+=40;
	}
*/
}

/* Set up the colormap */
void create_cmap(void)
{

	int i;
	BYTE red,green,blue;
	/*unsigned short rgb;*/
	unsigned long rgb;

	for(i=0; i< 256; i++)
	{
		red  = (BYTE)((colortable[i] & 0xff0000) >> 16);
		green = (BYTE)((colortable[i] & 0x00ff00) >> 8);
		blue = (BYTE)(colortable[i] & 0x0000ff);
		/*red=red/4;
		green=green/4;
		blue=blue/4;*/
		/*set_palette(i,red,green,blue);*/
		/*rgb = (red >> 3) << 11;
		rgb |= (green >> 2) << 5;
		rgb |= (blue >> 3) << 0;*/

		/* Converting to 5:5:5:1 */
		rgb = 0;
		rgb = (red >> 3) << 11;
		rgb |= (green >> 3) << 6;
		rgb |= (blue >> 3) << 1;
		palette_tmp[i] = rgb;
	}

	h_pal = GpPaletteCreate(256, (GP_PALETTEENTRY*)palette_tmp);
	GpPaletteSelect(h_pal);
	GpPaletteRealize();
/*	GpPaletteDelete(h_pal);
	h_pal = NULL;	*/

}

/* Create the main application shell */
static void create_window()
{

	int i;

	vwidth=tv_width*2;
	vheight=tv_height+tv_overscan;
	if (!app_data.debug) {
		init_keyboard();
		create_cmap();
	}
	for(i=0; i<256; i++) coltable[i]=1;
	coltable[0]=coltable[1]=coltable[2]=coltable[4]=0;
	coltable[8]=coltable[16]=coltable[32]=coltable[64]=0;
	coltable[128]=0;

}

void tv_off(void)
{
/*
	#ifdef VERBOSE
	  printf("Switching off...\n");
	#endif
	if (!app_data.debug) {
		Set_Old_Int9();
	}
	free(vscreen);
	setmode(TEXT_MODE);
*/
}

/* The main initialiser for the X stuff */
int tv_on()
{

	long i;

  	white=0;
	black=15;
/*
	if (!app_data.debug) {
		setmode(VGA256);
	}
*/
	create_window();
/*
	#ifdef VERBOSE
	   printf("OK\n  Allocating screen buffer...");
	#endif
*/
	vscreen=(BYTE *)malloc(64000);
	memset(vscreen,0,64000);
	if(!vscreen) {
/*		#ifdef VERBOSE
			printf("FAILED\n");
		#endif*/
	return(0);
	}
	return(1);

}

/* The graceful shut down for the X stuff */


/* Translates a 2600 color into is X pixel value */
/* BYTE tv_color(BYTE b)
{
	if(app_data.private)
	return (b >> 1);
	else
	return colors[(b >> 1)].pixel;
}  */

/* Displays the 2600 screen display */
/* Called from MEMORY.C - copies vbuffer to screen */
void tv_display(void)
{
/*	if (!app_data.debug) {*/
/*		put_image();*/
/*	}*/
	tv_counter++;

	/* Refresh */
	/*
	GpBitBlt(NULL,
				&gpDraw[nflip],
				0, 0,
				320, 240,
				vidbuf.ptbuffer,
				0, 0,
				320, 240);
	*/
	/*flip the lcd surcace over, makes it very smooth as we are always drawing on the
	one no one can see then flipping it over when its finished*/
	/*Tia_process(soundbuffer1, SOUND_BUFFERSIZE);*/
	if(tv_counter % app_data.rr == 0) {
		ExKey = GpKeyGet();
		GpSurfaceFlip(&gpDraw[nflip++]);	/* Flips buffers and waits for beam */
		nflip &= 0x01;
		/*GpPcmStop();
		GpPcmPlay(&soundbuffer1[0], SOUND_BUFFERSIZE, 1);*/
		if ((ExKey & GPC_VK_FL) && (ExKey & GPC_VK_FR)) {
			current_screen = SCREEN_MENU;
			/*StopEmuSound();*/
			/*GpPcmStop();*/
		}
	}
	#ifdef SW_FPSCOUNT
	frames_count++;		/* FPS Counter */
	#endif
}


/* Put one pixel to the image buffer. Used mainly for debugging */
/* UNUSED */
void tv_putpixel(int x, int y,BYTE value)
{
	BYTE *p;

	x=x<<1;
	p=vscreen+x+y*vwidth;
	*(p)=value;
	*(p+1)=value;
}

